/*
 *  @file   OsalPrint.c
 *
 *  @brief      Linux kernel Semaphore interface implementations.
 *
 *              This will have the definitions for kernel side printf
 *              statements and also details of variable printfs
 *              supported in existing implementation.
 *
 *
 *  ============================================================================
 *
 *  Copyright (C) {2008-2011} Texas Instruments, Incorporated
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the
 *  Free Software Foundation version 2.
 *  
 *  This program is distributed "as is" WITHOUT ANY WARRANTY of any kind,
 *  whether express or implied; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *  General Public License for more details
 *  
 *  Contact information for paper mail:
 *  Texas Instruments
 *  Post Office Box 655303
 *  Dallas, Texas 75265
 *  Contact information: 
 *  http://www-k.ext.ti.com/sc/technical-support/product-information-centers.htm?
 *  DCMP=TIHomeTracking&HQS=Other+OT+home_d_contact
 *  ============================================================================
 *  
 */



/* Standard headers */
#include <ti/syslink/Std.h>

/* OSAL and kernel utils */
#include <ti/syslink/utils/OsalPrint.h>

/* Linux specifc header files */
#include <stdarg.h>
#include <linux/kernel.h>


#if defined (__cplusplus)
extern "C" {
#endif

/*!
 *  @brief      printf abstraction at the kernel level.
 *
 *  @param      string which needs to be printed.
 *  @param      Variable number of parameters for printf call
 */
Void Osal_printf (char* format, ...)
{
    UInt32  bufCount     = 0 ;
    va_list args             ;
    char    buffer [512 ]    ;

    buffer[0] = '<';
    buffer[1] = '1';
    buffer[2] = '>';
    bufCount  =  3 ;

    va_start (args, format) ;
    vsprintf (buffer + bufCount, format, args);
    va_end   (args) ;

    printk (buffer) ;
}
#if defined (__cplusplus)
}
#endif /* defined (_cplusplus)*/
